"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const CommandList_1 = __importDefault(require("../Models/CommandList"));
const EMPTY_COMMAND = {
    AllowedChannels: [],
    AllowedRoles: [],
    AllowedUsers: [],
    Data: {}
};
class Processor {
    constructor(context, prefix = '!') {
        this.Context = context;
        this.Prefix = prefix;
        this.Loaded = false;
    }
    async LoadCommandsLocal(Commands) {
        const data = new CommandList_1.default();
        Object.keys(Commands).forEach((key) => {
            const { AllowedChannels, AllowedRoles, AllowedUsers } = EMPTY_COMMAND;
            data[key.toLowerCase()] = new Commands[key](AllowedChannels, AllowedRoles, AllowedUsers, false);
            console.log('[SUCCESS] Loaded Command', key);
        });
        return data;
    }
    async LoadCommandsFromDatabase(Commands) {
        const data = new CommandList_1.default();
        const databaseCommands = this.Context.State.Commands;
        Object.keys(Commands).forEach((key) => {
            const commandData = databaseCommands[key] || Object.assign({}, EMPTY_COMMAND, { Namespace: key });
            const { AllowedChannels, AllowedRoles, AllowedUsers, Data } = commandData;
            data[key.toLowerCase()] = new Commands[key](AllowedChannels, AllowedRoles, AllowedUsers, true);
            data[key.toLowerCase()].Data = Data;
            console.log('[SUCCESS] Loaded Command', key);
        });
        return data;
    }
    async LoadCommands(Commands, useDB) {
        if (this.Context.Loading === true) {
            const deferred = new Promise((resolve) => setTimeout(resolve, 300));
            await deferred;
        }
        if (useDB) {
            this.Commands = await this.LoadCommandsFromDatabase(Commands);
        }
        else {
            this.Commands = await this.LoadCommandsLocal(Commands);
        }
        return this.Commands;
    }
    async Handle(message) {
        try {
            if (this.Context.Loading === true || this.Loaded === false) {
                console.log('[WARNING] Message Received before Data is loaded, delaying');
                setTimeout(() => this.Handle(message), Processor.DELAY_TIMER);
                return;
            }
            if (message.content.indexOf(this.Prefix) !== 0) {
                return Promise.resolve('');
            }
            const name = message.content.split(' ').shift().substr(1);
            console.log('[MESSAGE] Running', name, message.content);
            return await this.Commands[name.toLowerCase()].Call(message);
        }
        catch (e) {
            console.error('[Fatal Error]', e);
            return Promise.reject(e);
        }
    }
}
Processor.DELAY_TIMER = 300;
exports.default = Processor;
